/**
 * @file led.c
 *
 * @brief This file contains led routines for the USBlini
 *        firmware project.
 *
 * @author Thomas Fischl (https://www.fischl.de/contact.html)
 * @copyright (c) 2021 Thomas Fischl
 * 
 * This file is part of the USBlini firmware project and is copyrighted by the
 * authors listed above. It is free for private or educational non-commercial use
 * (for a commercial license please contact the authors). It may not be
 * redistributed without the prior written consent of the authors.
 *
 * It is provided "as is" without warranty of any kind, either express or implied,
 * including without limitation any implied warranties of condition, uninterrupted
 * use, merchantability, fitness for a particular purpose, or non-infringement. In
 * no event shall the authors be liable for any direct or indirect damages arising
 * in any way out of the use of it.
 *
 */

#include "usblini.h"
#include "clock.h"
#include "led.h"

uint8_t state_laststamp;
uint8_t periodticker;

/**
 * @brief Initialize LED handling
 */
void led_init(void) {
    periodticker = 0;
    LED_PIN = 1;
}

/**
 * @brief Flash LED
 */
void led_flash(void) {
    
    // check if already flashing
    if (periodticker) return;
    
    periodticker = LED_FLASHPERIOD;
    LED_PIN = 0;
}

/**
 * @brief Timer interrupt routine 100 Hz
 */
inline void led_isr(void) {

    if (!periodticker) return;
    
    if (periodticker == LED_FLASHTIMEOFF) LED_PIN = 1;
    
    periodticker--;
    
}
