/**
 * @file lin.h
 *
 * @brief This file contains defintions for LIN bus functions for the USBlini
 *        firmware project.
 *
 * @author Thomas Fischl (https://www.fischl.de/contact.html)
 * @copyright (c) 2021 Thomas Fischl
 * 
 * This file is part of the USBlini firmware project and is copyrighted by the
 * authors listed above. It is free for private or educational non-commercial use
 * (for a commercial license please contact the authors). It may not be
 * redistributed without the prior written consent of the authors.
 *
 * It is provided "as is" without warranty of any kind, either express or implied,
 * including without limitation any implied warranties of condition, uninterrupted
 * use, merchantability, fitness for a particular purpose, or non-infringement. In
 * no event shall the authors be liable for any direct or indirect damages arising
 * in any way out of the use of it.
 *
 */
#ifndef LIN_H
#define	LIN_H

#define LIN_CHECKSUM_NONE 0
#define LIN_CHECKSUM_LIN1 1 // classic checksum, only data bytes
#define LIN_CHECKSUM_LIN2 2 // enhanced checksum, data bytes and pid

#define LIN_BAUDRATE_MIN 1200
#define LIN_BAUDRATE_MAX 20000
#define LIN_BAUDRATE_DEFAULT 19200

#define RECEIVESTATE_IDLE 0
#define RECEIVESTATE_ABAUD_START 1
#define RECEIVESTATE_ABAUD_SYNC 2
#define RECEIVESTATE_PID 3
#define RECEIVESTATE_DATA 4
#define RECEIVEBUFFER_SIZE 9

#define SENDBUFFER_SIZE 9
#define SENDSTATE_IDLE 0
#define SENDSTATE_BREAKSYNC 1
#define SENDSTATE_PID 2
#define SENDSTATE_DATA 3

#define REPORT_SOURCE_COMMON 0x00
#define REPORT_SOURCE_MASTER 0x10
#define REPORT_SOURCE_SLAVE 0x20
#define REPORT_SOURCE_USER 0x30

#define REPORT_TYPE_STATUS 0x00
#define REPORT_TYPE_ERROR 0x01
#define REPORT_TYPE_FRAME 0x02

#define ERRORID_NO_ECHO 0x01

struct frameinfo {
    uint8_t reportid;
    uint8_t pid;
    uint8_t length;
    uint8_t data[RECEIVEBUFFER_SIZE];
    uint16_t timestamp;
    uint16_t abaud;
};

void lin_init(void);
void lin_setBaudrate(uint16_t baudrate);
void lin_setConfiguredSPBRG(void);
void lin_setAutobaud(uint8_t active);
uint8_t lin_getSendState(void);
void lin_sendHeader(uint8_t id, uint8_t source);
void lin_sendDataWithHeader(uint8_t id, uint8_t * data, uint8_t length, uint8_t source);
void lin_sendData(uint8_t * data, uint8_t length, uint8_t source);
uint8_t lin_chReceived(void);
uint8_t lin_readByte(void);
inline void lin_timeout_isr(void);
uint8_t lin_getPID(uint8_t id);
uint8_t lin_getReceiveState(void);
void lin_waitFrameReceived(void);
struct frameinfo * lin_getFrameinfo(void);
void lin_autobaudSuspend(uint8_t suspended);
void lin_process(void);

#endif

